"""Top-level package for medstat."""

__author__ = """Pierre Monferrand"""
__version__ = '1.0.0'

from medstat.medstat import test_hypothesis, analyse_dataset


__all__ = [test_hypothesis, analyse_dataset]
